package com.ruoyi.web.controller.warehouse.warehouseBusiness;

import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.annotation.RepeatSubmit;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.ServletUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.utils.spring.SpringUtils;
import com.ruoyi.framework.web.service.TokenService;
import com.ruoyi.warehouseBusiness.domain.TWarehouseBills;
import com.ruoyi.warehouseBusiness.excel.StockTransfer;
import com.ruoyi.warehouseBusiness.service.ITWarehouseBillsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * 仓库调拨主表Controller
 *
 * @author ruoyi
 * @date 2020-12-11
 */
@RestController
@RequestMapping("/warehouseBusiness/stockTransfer")
public class TWarehouseStockTransferController extends BaseController {
    @Autowired
    private ITWarehouseBillsService itWarehouseBillsService;

    /**
     * 查询入库详情主表列表
     */
    @PreAuthorize("@ss.hasPermi('warehouseBusiness:stockTransfer:list')")
    @DataScope(deptAlias = "d", userAlias = "u")
    @GetMapping("/list")
    public TableDataInfo list(TWarehouseBills tWarehouseBills) {
        startPage();
        if(tWarehouseBills.getType().equals("1")){
            tWarehouseBills.setfBilltype("CKDB");
        }else{
            tWarehouseBills.setfBilltype("HWTG");
        }
        List<Map<String, Object>> list = itWarehouseBillsService.selectWarehouseBusinessList(tWarehouseBills);
        return getDataTable(list);
    }


    /**
     * 导出入库详情主表列表
     */
    @PreAuthorize("@ss.hasPermi('warehouseBusiness:stockTransfer:export')")
    @Log(title = "详情主表", businessType = BusinessType.EXPORT)
    @DataScope(deptAlias = "d", userAlias = "u")
    @GetMapping("/export")
    public AjaxResult export(TWarehouseBills tWarehouseBills) throws Exception {
        String exportName =null;
        if(tWarehouseBills.getType().equals("1")){
            tWarehouseBills.setfBilltype("CKDB");
            exportName="调拨";
        }else{
            tWarehouseBills.setfBilltype("HWTG");
            exportName="货物通关";
        }
        List<StockTransfer> list = itWarehouseBillsService.stockTransferExport(tWarehouseBills);
        ExcelUtil<StockTransfer> util = new ExcelUtil<>(StockTransfer.class);
        return util.exportExcel(list, exportName);
    }

    /**
     * 获取入库仓库详细信息
     */
    @PreAuthorize("@ss.hasPermi('warehouseBusiness:stockTransfer:edit')")
    @GetMapping(value = "/{fId}")
    @RepeatSubmit
    public AjaxResult getInfo(@PathVariable("fId") Long fId) {
        return AjaxResult.success(itWarehouseBillsService.selectTWarehousebillsById(fId));
    }

    /**
     * 入库暂存
     */
    @PreAuthorize("@ss.hasPermi('warehouseBusiness:stockTransfer:add')")
    @Log(title = "详情主表", businessType = BusinessType.INSERT)
    @PostMapping(value = "/add")
    @RepeatSubmit
    public AjaxResult add(@RequestParam("warehouseBills") String warehouseBills,
                          @RequestParam("tEnclosure") String tEnclosure,
                          @RequestParam("tCntr") String tCntr,
                          @RequestParam("warehousebillsfeesCr") String warehousebillsfeesCr,
                          @RequestParam("warehousebillsfeesDr") String warehousebillsfeesDr,
                          @RequestParam("warehousebillsitems") String warehousebillsitems,
                          @RequestParam("type") String type,
                          @RequestParam("mblnoStatus") String mblnoStatus) {
        String billsType =null;
        if(type.equals("1")){
             billsType = "CKDB";
        }else{
             billsType = "HWTG";
        }
        if (StringUtils.isNull(warehouseBills) || "{}".equals(warehouseBills)) {
            return AjaxResult.error("未找到仓库主表数据，请确认");
        }
        if (StringUtils.isEmpty(mblnoStatus)) {
            // 判断是提单号是否在一天当中出现第二次或以上
            AjaxResult result = itWarehouseBillsService.selectContainMblno(warehouseBills, billsType);
            String ajaxCode = result.get("code").toString();
            if ("500".equals(ajaxCode)) {
                return result;
            }
        }
        LoginUser loginUser = SpringUtils.getBean(TokenService.class).getLoginUser(ServletUtils.getRequest());
        return itWarehouseBillsService.insertTWarehousebills(warehouseBills, warehousebillsfeesCr,
                warehousebillsfeesDr, warehousebillsitems, tEnclosure, loginUser, tCntr, billsType);
    }

    /**
     * 打印作业单  状态 改为 -->  待入库
     * 生成 流水号 --> log 记录 20
     * waitWarehousing
     */
    @PreAuthorize("@ss.hasPermi('warehouseBusiness:inStock:add')")
    @Log(title = "详情主表", businessType = BusinessType.INSERT)
    @PostMapping(value = "/waitWarehousing")
    @RepeatSubmit
    public AjaxResult waitWarehousing(@RequestParam("warehouseBills") String warehouseBills,
                                      @RequestParam("tEnclosure") String tEnclosure,
                                      @RequestParam("tCntr") String tCntr,
                                      @RequestParam("warehousebillsfeesCr") String warehousebillsfeesCr,
                                      @RequestParam("warehousebillsfeesDr") String warehousebillsfeesDr,
                                      @RequestParam("warehousebillsitems") String warehousebillsitems,
                                      @RequestParam("type") String type
    ) {
        String billtystatus =null;
        if(type.equals("1")){
            billtystatus = "DB";
        }else{
            billtystatus = "TG";
        }
        Long billsType = 20L;
        // 获取当前的用户
        LoginUser loginUser = SpringUtils.getBean(TokenService.class).getLoginUser(ServletUtils.getRequest());
        return itWarehouseBillsService.waitWarehousing(warehouseBills, warehousebillsitems, tEnclosure,
                warehousebillsfeesCr, warehousebillsfeesDr, loginUser, tCntr, billsType, billtystatus);
    }

    /**
     *   装货      状态 改为 -->  装货中
     *                   log 记录 30
     *      ischargeCargo
     */
    @PreAuthorize("@ss.hasPermi('warehouseBusiness:outStock:add')")
    @Log(title = "详情主表", businessType = BusinessType.INSERT)
    @PostMapping(value = "/ischargeCargo")
    @RepeatSubmit
    public AjaxResult ischargeCargo(@RequestParam("warehouseBills") String warehouseBills,
                                    @RequestParam("tEnclosure") String tEnclosure,
                                    @RequestParam("tCntr") String tCntr,
                                    @RequestParam("warehousebillsfeesCr") String warehousebillsfeesCr,
                                    @RequestParam("warehousebillsfeesDr") String warehousebillsfeesDr,
                                    @RequestParam("warehousebillsitems") String warehousebillsitems) {
        Long billsType = 30L;
        // 获取当前的用户
        LoginUser loginUser = SpringUtils.getBean(TokenService.class).getLoginUser(ServletUtils.getRequest());
        return itWarehouseBillsService.waitWarehousing(warehouseBills, warehousebillsitems,
                tEnclosure, warehousebillsfeesCr, warehousebillsfeesDr, loginUser, tCntr, billsType,"1");
    }

    /**
     * 调拨提交
     */
    @PreAuthorize("@ss.hasPermi('warehouseBusiness:stockTransfer:add')")
    @Log(title = "详情主表", businessType = BusinessType.INSERT)
    @PostMapping(value = "/warehouseSubmission")
    @RepeatSubmit
    public AjaxResult warehouseSubmission(@RequestParam("warehouseBills") String warehouseBills,
                                          @RequestParam("tEnclosure") String tEnclosure,
                                          @RequestParam("tCntr") String tCntr,
                                          @RequestParam("warehousebillsfeesCr") String warehousebillsfeesCr,
                                          @RequestParam("warehousebillsfeesDr") String warehousebillsfeesDr,
                                          @RequestParam("warehousebillsitems") String warehousebillsitems,
                                          @RequestParam("type") String type) {
            String billsType =null;
            if(type.equals("1")){
                billsType = "CKDB";
            }else{
                billsType = "HWTG";
            }
        if (StringUtils.isNull(warehouseBills) || "{}".equals(warehouseBills)) {
            return AjaxResult.error("未找到仓库主表数据，请确认");
        }
        /*if (StringUtils.isNull(warehousebillsitems) || "[]".equals(warehousebillsitems)) {
            return AjaxResult.error("未找到仓库明细数据，请确认");
        }*/
        LoginUser loginUser = SpringUtils.getBean(TokenService.class).getLoginUser(ServletUtils.getRequest());
        return itWarehouseBillsService.warehouseSubmission(warehouseBills, warehousebillsfeesCr,
                warehousebillsfeesDr, warehousebillsitems, tEnclosure, loginUser, tCntr, billsType);
    }

    /**
     * 修改入库详情主表
     */
    @PreAuthorize("@ss.hasPermi('warehouseBusiness:stockTransfer:edit')")
    @Log(title = "详情主表", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TWarehouseBills tWarehouseBills) {
        return toAjax(itWarehouseBillsService.updateTWarehousebills(tWarehouseBills));
    }

    /**
     * 删除入库详情主表
     */
    @PreAuthorize("@ss.hasPermi('warehouseBusiness:stockTransfer:remove')")
    @Log(title = "详情主表", businessType = BusinessType.DELETE)
    @DeleteMapping("/{fIds}")
    @RepeatSubmit
    public AjaxResult remove(@PathVariable Long[] fIds) {
        return itWarehouseBillsService.deleteTWarehousebillsByIds(fIds);
    }

    @PreAuthorize("@ss.hasPermi('warehouseBusiness:stockTransfer:remove')")
    @Log(title = "详情主表", businessType = BusinessType.DELETE)
    @DeleteMapping("/queryRemove/{fId}")
    public AjaxResult queryRemove(@PathVariable("fId") Long fId) {
        return itWarehouseBillsService.queryRemove(fId);
    }

    /**
     * 撤销入库信息
     */
    @PreAuthorize("@ss.hasPermi('warehouseBusiness:stockTransfer:revoke')")
    @Log(title = "详情主表", businessType = BusinessType.UPDATE)
    @GetMapping("/get/{fId}")
    @RepeatSubmit
    public AjaxResult revoke(@PathVariable("fId") Long fId) {
        String billsType = "CKDBRevoke";
        // 获取当前的用户
        LoginUser loginUser = SpringUtils.getBean(TokenService.class).getLoginUser(ServletUtils.getRequest());
        return itWarehouseBillsService.revokeTWarehousebillsById(fId, loginUser, billsType);
    }

    /**
     * 撤销入库信息
     */
    @PreAuthorize("@ss.hasPermi('warehouseBusiness:stockTransfer:edit')")
    @Log(title = "详情主表", businessType = BusinessType.UPDATE)
    @GetMapping("/withdrawById/{fId}")
    @RepeatSubmit
    public AjaxResult withdrawById(@PathVariable("fId") Long fId) {
        // 查询是否对账、收费、付费
        int result = itWarehouseBillsService.derecognition(fId);
        if(result == 1){
            return AjaxResult.error("撤销请核失败，财务已对账");
        } else if (result == 2){
            return AjaxResult.error("撤销请核失败，财务已收费");
        } else if (result == 3){
            return AjaxResult.error("撤销请核失败，财务已付费");
        }
        String billsType = "CKDBRevoke";
        // 获取当前的用户
        LoginUser loginUser = SpringUtils.getBean(TokenService.class).getLoginUser(ServletUtils.getRequest());
        return itWarehouseBillsService.revokeTWarehousebillsById(fId, loginUser, billsType);
    }

    /**
     * 撤销入库
     */
    @PreAuthorize("@ss.hasPermi('warehouseBusiness:stockTransfer:add')")
    @Log(title = "详情主表", businessType = BusinessType.INSERT)
    @PostMapping(value = "/updateCredit")
    public AjaxResult deleteCredit(@RequestParam("warehouseBills") String warehouseBills,
                                   @RequestParam("tEnclosure") String tEnclosure,
                                   @RequestParam("tCntr") String tCntr,
                                   @RequestParam("warehousebillsfeesCr") String warehousebillsfeesCr,
                                   @RequestParam("warehousebillsfeesDr") String warehousebillsfeesDr,
                                   @RequestParam("warehousebillsitems") String warehousebillsitems,
                                   @RequestParam("type") String type) {
        String billsType =null;
        if(type.equals("1")){
             billsType = "CKDBRevoke";
        }else{
             billsType = "HWTGRevoke";
        }
        // 获取当前的用户
        LoginUser loginUser = SpringUtils.getBean(TokenService.class).getLoginUser(ServletUtils.getRequest());
        return itWarehouseBillsService.revokeTWarehousebillsById1(warehouseBills, warehousebillsitems,
                tEnclosure, warehousebillsfeesCr, warehousebillsfeesDr, loginUser, tCntr, billsType);
    }

    /**
     * 调拨确认   状态 改为 -->  已入库
     */
    @PreAuthorize("@ss.hasPermi('warehouseBusiness:stockTransfer:add')")
    @Log(title = "详情主表", businessType = BusinessType.INSERT)
    @PostMapping(value = "/addCredit")
    @RepeatSubmit
    public AjaxResult addCredit(@RequestParam("warehouseBills") String warehouseBills,
                                @RequestParam("tEnclosure") String tEnclosure,
                                @RequestParam("tCntr") String tCntr,
                                @RequestParam("warehousebillsfeesCr") String warehousebillsfeesCr,
                                @RequestParam("warehousebillsfeesDr") String warehousebillsfeesDr,
                                @RequestParam("warehousebillsitems") String warehousebillsitems,
                                @RequestParam("type") String type) {
            String billsType =null;
            if(type.equals("1")){
                billsType = "CKDB";
            }else{
                billsType = "HWTG";
            }
        if (StringUtils.isNull(warehouseBills) || "{}".equals(warehouseBills)) {
            return AjaxResult.error("未找到主表信息，请确认");
        }
        if (StringUtils.isNull(warehousebillsitems) || "[]".equals(warehousebillsitems)) {
            return AjaxResult.error("未找到明细信息，请确认");
        }
        // 获取当前的用户
        LoginUser loginUser = SpringUtils.getBean(TokenService.class).getLoginUser(ServletUtils.getRequest());
        return itWarehouseBillsService.updateCredit(warehouseBills, warehousebillsitems, tEnclosure, warehousebillsfeesCr,
                warehousebillsfeesDr, loginUser, tCntr, billsType);
    }

    /**
     *   明細流水号
     */
    @PostMapping(value = "/serialNumber")
    public AjaxResult serialNumber() {
        String billsType = "CKDB";
        return itWarehouseBillsService.serialNumber(billsType);
    }

    /**
     *  根据id更新费用审核状态
     * @param id
     * @return
     */
    @GetMapping(value = "/feeReview")
    @RepeatSubmit
    public AjaxResult feeReviewSubmit(@RequestParam("id") Long id) {
        return itWarehouseBillsService.feeReviewSubmit(id);
    }

    /**
     *  根据id撤销已费用审核状态
     * @param id
     * @return
     */
    @GetMapping(value = "/revokefeeReview")
    @RepeatSubmit
    public AjaxResult revokefeeReview(@RequestParam("id") Long id) {
        return itWarehouseBillsService.revokefeeReview(id);
    }

}
